/*------------------------------------------------------------------------
  Neuricam SpA   - Trento - Italy
  http://www.neuricam.com
  email:  techsupport@neuricam.com , info@neuricam.com
--------------------------------------------------------------------------
  Filename: data_collection.cpp
--------------------------------------------------------------------------
  Project:       <PCN-1001>
  Author(s):     <Roberto Doriguzzi> <Roberto Corradi>
  First release: <Jan 2007>
  Description:   <PCN-1001 image server>
-------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <pthread.h>
#include <termios.h>
#include <math.h>
#include <linux/watchdog.h>
#include <linux/videodev.h>
#include <getopt.h>
#include "default_parms.h"
#include <asm/io.h>

//#define DEBUG

unsigned char sys_topology[MAX_ZONE][MAX_SYSTEM_IN_ZONE]; // zona[] sistema[]
unsigned long people[MAX_SYSTEM][2]; // sistema [] in-out[]
char records[MAX_SYSTEM][68]; // sistema [] stringa[]
unsigned long people_in_zone[MAX_ZONE][2]; // zona [] in-out[]
unsigned long old_people_in_zone[MAX_ZONE][2]; // zona [] in-out[]
char door_status[MAX_SYSTEM];
char old_door_status[MAX_SYSTEM];
bool door_status_in_zone_open[MAX_ZONE];
bool old_door_status_in_zone_open[MAX_ZONE];
bool door_status_in_zone_close[MAX_ZONE];
bool old_door_status_in_zone_close[MAX_ZONE];
char ct_filename_zone[MAX_SYSTEM][128]; 	// concentrator log
char log_filename_zone[MAX_SYSTEM][128]; 	// concentrator log

bool answer_msg;
unsigned int tmout;

char working_dir[128] = WORK_DIR;	// working directory
char pm_filename[128]; 	// parameters file path
char ct_filename[128]; 	// concentrator log
char tp_filename[128]; 	// configuration topology

int thread_status;			// mainloop thread status
bool wd_check;
int watchdog;
int opto1;
int opto2;
int opto3;
int opto4;
unsigned char opto_in[4];
unsigned char stauts_opto;
bool thread_sleep;

pthread_t serloopttyZ3;
pthread_mutex_t rdlock;		// records lock
pthread_mutex_t mainlock;	// concurrency between function main_loop and the main thread
				// (reset_counters function and "serial" command)
pthread_t mloop;
pthread_t inloop;
pthread_t wdloop;		// watchdog loop
/**************** serial port variables **********************************/
int ttyZ3;
struct termios newtio;
/*******************  parameters *****************************************/

char parm_names[256][PARM_STR_LEN] = {
"serial_id",
"serial_br",
"serial_db",
"serial_pr",
"serial_sb",
"timer_data"
};

unsigned short parm_values[256];
unsigned short default_values[256] = {
SERIAL_ID,
SERIAL_BR,
SERIAL_DB,
SERIAL_PR,
SERIAL_SB,
TIMER_DATA
};

/****************************   gpios strucures   ************************/
struct reg_info {
   char *name;
   unsigned int addr;
   int shift;
   unsigned int mask;
   char type;
   char *desc;
};
static struct reg_info regs[] = {
{ "THR_in_LSR", 0x0800030A,  6, 0x00000001, 'd', "transmitter empty" }
};

/******************   extern variables   ************************/
//extern unsigned char Bkgvec[NN], Pasvec[NN];

/******************   initialization functions    ************************/
void var_init();
int init_io();
void PrintMenu();

/****************  loops functions ***************************************/
void *main_loop(void *fd);
void *ser_loopttyZ3(void *arg);
void mainloop_enable(bool enable);
void *watchdog_loop(void *arg);
void *input_loop(void *arg);
/*************************  I/O functions    *****************************/
void load_parms(void);
void load_sys_topology(void);
int load_default_parms(void);
int save_parms(char *name,unsigned short value);
unsigned short get_parms(char *name);
int set_parms(char *name,unsigned short value);
int write_parms(char *name,unsigned short value);
int get_mem(unsigned int addr);
int put_mem(unsigned int addr, unsigned int val);
int get_gpio(char *name);
int set_gpio(char *name,unsigned int val);
int read_opto();
int init_opto();
void SetDateTime();
/****************  serial_port functions *********************************/
void reset_serial(int fd);
int set_serial(int fd,unsigned long baudrate,unsigned long bitesize,
	        unsigned long parity, unsigned long stopbits);

int enable_port(unsigned short value);
int SNP_reply(unsigned char dest,unsigned char recipient,char *buffer,char *args,unsigned int args_size,int ttyZ);
int SNP_Send(unsigned char dest,char *command,char *arg,int arg_size,int ttyZ);
int SNP_Recv(char *buf,unsigned char *sender,unsigned char *recipient,int ttyZ);

int BuildSNPCommand(char *buf,unsigned char src,unsigned char dest,char tpn,char pn,short datalen,char *data);
void CalcCrc16Block(char *pBlock, unsigned short Number, short *pCrc);
unsigned long get_arg(char *arg,int arg_size,int pos,int size);
int read_port(int fd,char *buf,int len);
int write_port(int fd,char *buf,int len);

/**********************************************************************/

int main(int argc, char **argv)
{   
    var_init();				// setting global variables
    if(init_io() < 0) exit(0);		// initializing  devices
    load_parms();			// loading saved parameters
    load_sys_topology();
    SetDateTime();
    
    char value=1;
    SNP_Send(0xff,"enable_pc",(char*)&value,sizeof(value),ttyZ3);     // abilito il conteggio,mi metto in condizione nota
    stauts_opto = read_opto();
    printf("stato iniziale opto: %d\n",stauts_opto);
    for(int i=0;i<255;i++)  
    {
      for(int u=0;u<255;u++)
      {
        if(sys_topology[i][u]==2 || sys_topology[i][u]==3 || sys_topology[i][u]==4 || sys_topology[i][u]==5)
        {  
        unsigned char val = ((stauts_opto >> (sys_topology[i][u]-2)) & 0x01);

        tmout=200;  
        while (--tmout && answer_msg==false)
          usleep(1);  // va bene forse 1
        #ifdef DEBUG
        if(!tmout)
           printf("Time out error. System don't respond \n");	  
        #endif  

        #ifdef DEBUG
        printf("Send enable_pc %d to ID %d \n",val,sys_topology[i][u]);
        #endif       
        SNP_Send(sys_topology[i][u],"enable_pc",(char*)&val,sizeof(unsigned char),ttyZ3);
        answer_msg=false;
        tmout=200;  
        while (--tmout && answer_msg==false)
        usleep(1);  
     //   #ifdef DEBUG
        if(!tmout)  
           printf("Time out error. System %d don't respond message enable_pc %d !\n",sys_topology[i][u],val);	  
     //   #endif   
        door_status[sys_topology[i][u]]=val;
        old_door_status[sys_topology[i][u]]=val;
        }
      }
    }  
    


    // ----------- starting threads and mutex -------------//
    pthread_create (&wdloop, NULL, watchdog_loop, NULL); //watchdog loop

    mainloop_enable(1); //starting the acquisition thread
    pthread_mutex_init(&mainlock,NULL);

    pthread_create (&inloop, NULL, input_loop, NULL);
    //-----------------------------------------------------//
 

    while(1)
    {
    char input_char[5];
    char command [256];
    char arg;
    PrintMenu();
    scanf("%s",input_char);
    switch(input_char[0])
      {
      case 'r':
          mainloop_enable(0);
          printf("Counters Reset !!\n");
          SNP_Send(0xff,"reset",NULL,0,ttyZ3);
          mainloop_enable(1);
          break;
      case 'd':
          mainloop_enable(0);
          printf("Delete log files !!\n");
          for(int u=0;u<255;u++)
          {
            if(sys_topology[u][0]!=0)
            {  
            sprintf(command,"rm %s",log_filename_zone[u]);
            system(command);
            sprintf(command,"rm %s",ct_filename_zone[u]);
            system(command);
            }
          }
          mainloop_enable(1);
          break;
      case 'o':
          mainloop_enable(0);
          if(input_char[1]!='2' && input_char[1]!='3' && input_char[1]!='4' && input_char[1]!='5')
          {
            arg =1;

            tmout=200;  
            while (--tmout && answer_msg==false)
              usleep(1);  // va bene forse 1
            #ifdef DEBUG
            if(!tmout)
                 printf("Time out error. System %d don't respond \n",sys_topology[i][u]);	  
            #endif  

            SNP_Send(atoi(&input_char[1]),"enable_pc",(char*)&arg,sizeof(arg),ttyZ3);
            printf("inviato enable_pc al system %d con %d!!\n",atoi(&input_char[1]),arg);
            answer_msg=false;

            tmout=200;  
            while (--tmout && answer_msg==false)
              usleep(1);  // va bene forse 1
        //    #ifdef DEBUG
            if(!tmout)
                 printf("Time out error. System %d don't respond messagge enable_pc with val=1 !\n",atoi(&input_char[1]));	  
        //    #endif  
          }
          mainloop_enable(1);
          break;
      case 'c':
          mainloop_enable(0);
          if(input_char[1]!='2' && input_char[1]!='3' && input_char[1]!='4' && input_char[1]!='5')
          {
            arg =0;

            tmout=200;  
            while (--tmout && answer_msg==false)
              usleep(1);  // va bene forse 1
            #ifdef DEBUG
            if(!tmout)
                 printf("Time out error. System %d don't respond \n",sys_topology[i][u]);	  
            #endif  

            SNP_Send(atoi(&input_char[1]),"enable_pc",(char*)&arg,sizeof(arg),ttyZ3);
            printf("inviato enable_pc al system %d con %d!!\n",atoi(&input_char[1]),arg);
            answer_msg=false;

            tmout=200;  
            while (--tmout && answer_msg==false)
              usleep(1);  // va bene forse 1
        //    #ifdef DEBUG
            if(!tmout)
                printf("Time out error. System %d don't respond message enable_pc with val=0!\n",atoi(&input_char[1]));	  
        //    #endif  
          }
          mainloop_enable(1);
          break;
      case 'u':
          PrintMenu();
          break;
      case 'e':
          printf("Exit \n");
          close(watchdog);
          close(opto1);
          close(opto2);
          close(opto3);
          close(opto4);
          pthread_cancel(mloop);
          pthread_cancel(inloop);
          pthread_mutex_destroy(&mainlock);
          return 0;
          break; 
      default:
          break;
      }
    }

    return 0;
}
/******************* initializing variables ********************************/
void var_init()
{
    sprintf(pm_filename,"%s%s",working_dir,PM_FILENAME);
    sprintf(ct_filename,"%s%s",working_dir,CT_FILENAME);
    sprintf(tp_filename,"%s%s",working_dir,TP_FILENAME);
    for(int i=0;i<MAX_SYSTEM;i++)
        sprintf(ct_filename_zone[i],"%s%s%d%s%s",working_dir,"Zone_",i,"_",CT_FILENAME);
    for(int i=0;i<MAX_SYSTEM;i++)
        sprintf(log_filename_zone[i],"%s%s%d%s%s",working_dir,"Log_Zone_",i,"_",CT_FILENAME);
 
    ttyZ3 = -1;			//serial port loop status = not active
    serloopttyZ3 = 0;			//serial port thread init
    thread_status = MAINLOOP_STOP;	//mainloop is not running

    for(int i=0;i<255;i++)
      {
      people[i][0]=0;
      people[i][1]=0;
      old_people_in_zone[i][0]=0;
      old_people_in_zone[i][1]=0;
      people_in_zone[i][0]=0;
      people_in_zone[i][1]=0;
      door_status[i]=1;
      old_door_status[i]=1;
      door_status_in_zone_open[i]=1;
      old_door_status_in_zone_open[i]=1;
      door_status_in_zone_close[i]=1;
      old_door_status_in_zone_close[i]=1;
      }
    wd_check = 0;			// watchdog check 
    opto1 = -1;
    opto2 = -1;;
    opto3 = -1;;
    opto4 = -1;;
    opto_in[0]=1;
    opto_in[1]=1;
    opto_in[2]=1;
    opto_in[3]=1;
    stauts_opto=0x0f;
    thread_sleep=false;
}

/***************************************************************************/
/************ initializing peripherals  ************************************/
/***************************************************************************/
int init_io()
{

    opto1 = open("/proc/sys/dev/avl_light/gpio/in_1",O_RDONLY);
    if (opto1 < 0) 
    {
      printf("Warning: cannot open /proc/sys/dev/avl_light/gpio/in_1.\n");
      return -1;
    }

    opto2 = open("/proc/sys/dev/avl_light/gpio/in_2",O_RDONLY);
    if (opto2 < 0) 
    {
      printf("Warning: cannot open /proc/sys/dev/avl_light/gpio/in_2.\n");
      return -1;
    }

    opto3 = open("/proc/sys/dev/avl_light/gpio/in_3",O_RDONLY);
    if (opto3 < 0) 
    {
      printf("Warning: cannot open /proc/sys/dev/avl_light/gpio/in_3.\n");
      return -1;
    }
    opto4 = open("/proc/sys/dev/avl_light/gpio/in_4",O_RDONLY);
    if (opto4 < 0) 
    {
      printf("Warning: cannot open /proc/sys/dev/avl_light/gpio/in_4.\n");
      return -1;
    }


    watchdog = open("/dev/watchdog",O_WRONLY);
    if (watchdog < 0) 
    {
      printf("Warning: cannot open /dev/watchdog.\n");
      printf("Warning: watchdog disabled.\n");
    }
    else
    {
      int timeout;
       
      timeout = 30;	// 30 seconds
      ioctl(watchdog, WDIOC_SETTIMEOUT, &timeout);
       
      write(watchdog, "\0", 1);
    }
    return 0;
}

/******************* print menu ********************************/
void SetDateTime()
{ 
  int lower_ID=255;
  for(int i=0;i<255;i++)
     for(int u=0;u<255;u++)

         if(sys_topology[i][u]<lower_ID & sys_topology[i][u]!=0) 
           lower_ID = sys_topology[i][u];

//  printf("ID lower %d\n",lower_ID);
  tmout=200;  
  while (--tmout && answer_msg==false)
     usleep(1);  // va bene forse 1
  #ifdef DEBUG
  if(!tmout)
    printf("Time out error. System %d don't respond \n",sys_topology[i][u]);	  
  #endif  

  SNP_Send(lower_ID,"gdatetime",NULL,0,ttyZ3);
 // printf("inviato gdatetime al system %d !!\n",lower_ID);
  answer_msg=false;

  tmout=200;  
  while (--tmout && answer_msg==false)
      usleep(1);  // va bene forse 1
//  #ifdef DEBUG
  if(!tmout)
      printf("Time out error. System %d don't respond message gdatetime!\n",lower_ID);	  
//  #endif  
}

/******************* print menu ********************************/
void PrintMenu()
{
  system("clear");
  for(int i=0;i<255;i++)
     if(sys_topology[i][0]!=0)
     {
       printf("Zone: %d IDs:",i);
       for(int u=0;u<255;u++)
          if(sys_topology[i][u]!=0) 
             printf(" %d ",sys_topology[i][u]);  
       printf("\n");
     }
  for(int i=0;i<255;i++)
    for(int u=0;u<255;u++)
       if(sys_topology[i][u]!=0)
       {
          if(door_status[sys_topology[i][u]]==0)
            printf("Sys. ID: %d -> Door Close",sys_topology[i][u]);
          else
            printf("Sys. ID: %d -> Door Open",sys_topology[i][u]);
          printf("\n");
       }
 
  printf("\n\n");
  printf(" ***************************************************************************\n");
  printf(" ********* Data collection menu (imgserver version 1.2.0 or higher)*********\n");
  printf(" ***************************************************************************\n");
  printf(" **	r) Reset counters						  **\n");
  printf(" **	d) Delete all the log files					  **\n");
  printf(" **	c) Close door (c + system ID, no for IDs 2,3,4,5)		  **\n");
  printf(" **	o) Open door (o + system ID, no for IDs 2,3,4,5)		  **\n");
  printf(" **	u) Update door status in menu					  **\n");
  printf(" **	e) Exit program							  **\n");
}


/***************************************************************************/
#include "loops.cpp"
#include "io.cpp"
#include "serial_port.cpp"
/***************************************************************************/
