/*------------------------------------------------------------------------
  Neuricam SpA   - Trento - Italy
  http://www.neuricam.com
  email:  techsupport@neuricam.com , info@neuricam.com
--------------------------------------------------------------------------
  Filename: io.cpp
--------------------------------------------------------------------------
  Project:       <PCN-1001>
  Author(s):     <Roberto Doriguzzi> <Roberto Corradi>
  First release: <Jan 2007>
  Description:   <PCN-1001 image server, various input/output functions>
-------------------------------------------------------------------------*/

/******************************************************************/
/*** loading parameters from a text file **************************/
/******************************************************************/
void load_parms(void)
{//carica parametri configurazione
  FILE *parms;
  char name[PARM_STR_LEN];
  char svalue[PARM_STR_LEN];
  unsigned short value;
  int ret;
  
  load_default_parms();

  if((parms = fopen(pm_filename,"r")) == NULL) return; 
  else
  {
    do
    {
      ret = fscanf(parms,"%s ",name);
      if((get_parms(name) != 0xFFFF) && ret != EOF)	// checking if the parameter is valid
      {
        fscanf(parms,"%s",svalue);
        value = strtol(svalue,NULL,16);
        set_parms(name,value);				// saving parameters in memory
        write_parms(name,value);			// writing parameters on devices
      }
    }
    while(ret != EOF);
  }

  enable_port(3);
  fclose(parms);
}

/******************************************************************/
/*** loading parameters from a text file **************************/
/******************************************************************/
void load_sys_topology(void)
{//carica parametri topologia
  FILE *tp_file;
  char name[PARM_STR_LEN];

  int ret;

  int zone_ID=0;
  int sys_number=0;
  int sys_ID=0;
  char filename[256];

  if((tp_file = fopen(tp_filename,"r")) == NULL) return;
  else
  {
    do
    {
      ret = fscanf(tp_file,"%s ",name);
      if((strcmp(name,"zone") == 0) && ret != EOF)
      {
        fscanf(tp_file,"%d",&zone_ID);
        printf("zone %d \n",zone_ID);
        ret = fscanf(tp_file,"%s",name);
        if((strcmp(name,"sys_number") == 0) && ret != EOF)
        {
          fscanf(tp_file,"%d",&sys_number);
          printf("sys_number %d \n",sys_number);
          ret = fscanf(tp_file,"%s",name);
          if((strcmp(name,"IDs") == 0) && ret != EOF)
          {
          for(int i=0;i<sys_number;i++)
            {
              ret = fscanf(tp_file,"%d",&sys_ID);
              printf("%d \n",sys_ID);
              sys_topology[zone_ID][i]= sys_ID;
            }
          }
        }
        fscanf(tp_file,"%s ",name);
        if((strcmp(name,"log_filename1") == 0) && ret != EOF)
        {
          fscanf(tp_file,"%s ",filename);
          strcpy(ct_filename_zone[zone_ID],filename);
          printf("%s\n",ct_filename_zone[zone_ID]);
        }
        fscanf(tp_file,"%s ",name);
        if((strcmp(name,"log_filename2") == 0) && ret != EOF)
        {
          fscanf(tp_file,"%s ",filename);
          strcpy(log_filename_zone[zone_ID],filename);
          printf("%s\n",log_filename_zone[zone_ID]);
        }
      }
    }
    while(ret != EOF);
  }
 
  fclose(tp_file);
}

/******************************************************************/
/*** loading default parameters ***********************************/
/******************************************************************/
int load_default_parms(void)
{
  int i;
  int ret = 0;
  
  memcpy(parm_values,default_values,sizeof(default_values));
  for(i=0;(strlen(parm_names[i]) && (i < 256));i++)
  {
    if(write_parms(parm_names[i],parm_values[i]) < 0)
    {
      printf("Error loading default parameter: %s\n",parm_names[i]);
      ret = -1;
    }
  }
  return ret;
}
/******************************************************************/
/*** saving parameters to a text file *****************************/
/******************************************************************/
int save_parms(char *name,unsigned short value)
{
  FILE *out;
  char sname[PARM_STR_LEN];
  char string[PARM_STR_LEN];
  char command[32];
  char ch;
  int i;
  long pos,len;
  
  if(set_parms(name,value) < 0) return -1;	//wrong parameter

  sprintf(command,"/bin/touch %s",pm_filename);	// creating the parameters file if it does not esist
  system(command);
  
  out = fopen(pm_filename,"r+");
  if(!out) {printf("Unable to open the file: %s\n",pm_filename); return -1;}
  else
  {
    while(!feof(out))
    { 
      pos = ftell(out);
      fscanf(out,"%s ",sname);
     
      if(!strcmp(name,sname))
      {
        fseek(out,pos,SEEK_SET);
        sprintf(string,"%s 0x%x",sname,value);
        len = strlen(string);
        for(i=len;i<PARM_STR_LEN-1;i++) string[i] = ' ';  //deleting unused chars
        string[PARM_STR_LEN-1] = '\0';
        fputs(string,out);
        fputc('\n',out);
        fclose(out);
        return 0;
      }
      do ch = fgetc(out);			//going to the end of the line
      while(ch != '\n' && !feof(out)) ;
    }
  }   
  
  
  sprintf(string,"%s 0x%x",name,value);
  len = strlen(string);
  for(i=len;i<PARM_STR_LEN-1;i++) string[i] = ' ';	//deleting unused chars
  string[PARM_STR_LEN-1] = '\0';
  fputs(string,out);
  fputc('\n',out);
    
  fclose(out);
  return 0;
}
/******************************************************************/
/*****************  get parm values from memory  ******************/
/******************************************************************/
unsigned short get_parms(char *name)
{
  int i;
  
  for(i=0;(strlen(parm_names[i]) && (i < 256));i++)
  {
    if(!strcmp(parm_names[i],name))
      return parm_values[i];
  }
  return 0xFFFF;
}
/******************************************************************/
/*******************  set parm values to memory  ******************/
/******************************************************************/
int set_parms(char *name,unsigned short value)
{
  int i;
  
  for(i=0;(parm_names[i] && (i < 256));i++)
  {
    if(!strcmp(parm_names[i],name))
    {
      parm_values[i] = value;
      return 0;
    }
  }
  return -1;
}
/******************************************************************/
/****************  write parm values to devices  ******************/
/******************************************************************/
int write_parms(char *name,unsigned short value)
{  //i tuoi parametri
  printf("Write parms %s=%d\n",name,value);

  if(strcmp(name,"serial_id")==0)
  {
    return 0;  	  
  }

  if(strncasecmp(name,"serial",6)==0)
  {
    set_serial(ttyZ3,get_parms("serial_br"),get_parms("serial_db"),
      			     get_parms("serial_pr"),get_parms("serial_sb"));
    return 0;  	       
  }  

  if(strcmp(name,"timer_data")==0)
  {
    return 0;  	       
  }

  return -1;
}


/***********************************************************************************/

int get_gpio(char *name)
{
  int i,data;
  int n=sizeof(regs)/sizeof(struct reg_info);

  for (i=0; i<n; i++) 
  {
     if (strstr(regs[i].name, name))
     {
        data = get_mem(regs[i].addr); 
        return (data >> regs[i].shift) & regs[i].mask;
     }
  }
  return -1;
}
/***********************************************************************************/

int set_gpio(char *name,unsigned int val)
{
   int i;
   unsigned int mem;
   int found=0;
   int count=0;
   int n=sizeof(regs)/sizeof(struct reg_info);

   for (i=0; i<n; i++) 
   {
      if (strcmp(regs[i].name, name)==0) 
      {
         found = i;
         count++;
      }
   }
   if (count!=1) 
   {
      exit(1);
   }
   
   mem = get_mem(regs[found].addr);
   mem &= ~(regs[found].mask << regs[found].shift);
   val &= regs[found].mask;
   mem |= val << regs[found].shift;
   put_mem(regs[found].addr, mem);  
   
   return 0; 
}
/***********************************************************************************/

int get_mem(unsigned int addr)
{
   void *map, *regaddr;
   int fd;
   unsigned int val;


   fd = open("/dev/mem", O_RDWR | O_SYNC);
   if (fd<0) 
   {
     perror("open(\"/dev/mem\")");
     return -1;
   }

   map = mmap(0, 
              MAP_SIZE, 
              PROT_READ | PROT_WRITE, 
              MAP_SHARED, 
              fd, 
              addr & ~MAP_MASK 
             );

   if (map < 0 ) 
   {
       perror("mmap()");
       exit(1);
   }
   regaddr = (unsigned char *)map + (addr & MAP_MASK);
   
   val = *(unsigned int *) regaddr;
   
   munmap(map,MAP_SIZE);
   close(fd);

   return val;
}
/***********************************************************************************/
int put_mem(unsigned int addr, unsigned int val)
{
   void *map, *regaddr;
   static int fd = -1;

   fd = open("/dev/mem", O_RDWR | O_SYNC);
   if (fd < 0) 
   {
     perror("open(\"/dev/mem\")");
     return -1;
   }
   
   map = mmap(0, 
              MAP_SIZE, 
              PROT_READ | PROT_WRITE, 
              MAP_SHARED, 
              fd, 
              addr & ~MAP_MASK 
             );
   if (map == (void*)-1 ) {
       perror("mmap()");
       exit(1);
   }

   regaddr = (unsigned char *)map + (addr & MAP_MASK);
    
   *(unsigned int *) regaddr = val;
   printf("write 0x%x 0x%x\n",addr,val);
   munmap(map,MAP_SIZE);
   close(fd);
   
   return 0;
}


